#! /usr/bin/env bash

trap "exit" INT

# https://github.com/betars/Face-Resources
# https://github.com/polarisZhao/awesome-face#-datasets
# https://github.com/jian667/face-dataset
# https://www.wikiwand.com/en/List_of_datasets_for_machine-learning_research#/Object_detection_and_recognition
# https://www.wikiwand.com/en/List_of_datasets_for_machine-learning_research#/Other_images
# https://www.face-rec.org/databases/

FD_HOME="$(realpath $(dirname $0))"
MAIN="${FD_HOME}/data/main/"
ALT="${FD_HOME}/data/alt/"


setupWD() {
	mkdir -p ${ALT}/testing/testing/pos/
	mkdir -p ${ALT}/testing/testing/neg/
}


checkPackages() {
	if [[ $(uname -s) == "Darwin" ]]
	then
		echo "Ensure your home directory is writable if you are running Catalina or this script will fail (https://stackoverflow.com/questions/58034955/read-only-file-system-when-attempting-mkdir-data-db-on-mac)"
		sleep 10

		brew list > /tmp/brewlist
		if ! grep "^julia$" /tmp/brewlist > /dev/null 2>&1
		then
			echo "Downloading Julia"
			brew cask install julia
		fi
		brew list > /tmp/brewlist
        if ! grep "^wget$" /tmp/brewlist > /dev/null 2>&1
        then
            echo "Downloading wget"
            brew install wget
        fi
		brew list > /tmp/brewlist
        if ! grep "^coreutils$" /tmp/brewlist > /dev/null 2>&1
        then
            echo "Downloading coreutils"
            brew install coreutils
        fi
	else
		echo "Please ensure Julia, wget, and coreutils is downloaded and in your path."
		sleep 10
	fi

	echo "Downloading dependencies"
	julia -E 'import Pkg; fd_home = dirname(@__FILE__); Pkg.activate(fd_home); Pkg.instantiate(); Pkg.activate(joinpath(fd_home, "examples")); Pkg.instantiate(); Pkg.activate(joinpath(fd_home, "test")); Pkg.instantiate()'
}

FD_HOME="$(realpath $(dirname $0))"
MAIN="${FD_HOME}/data/main/"
ALT="${FD_HOME}/data/alt/"

obtainDatasetAlt() {
	echo "Downloading alternative face detection training data"
	[[ -d ${FD_HOME}/face-detection-data/ ]] && rm -rf ${FD_HOME}/face-detection-data/
	[[ -d ${ALT}/pos/ ]] && rm -rf ${ALT}/pos/
	[[ -d ${ALT}/neg/ ]] && rm -rf ${ALT}/neg/
	
	cd ${FD_HOME}/ && \
		git clone https://github.com/OlegTheCat/face-detection-data && \
		mv ${FD_HOME}/face-detection-data/pos/ ${ALT}/ && \
		mv ${FD_HOME}/face-detection-data/neg/ ${ALT}/ && \
		rm -rf ${FD_HOME}/face-detection-data/
}


obtainDatasetMain() {
	echo "Dowloading training data"
	[[ -d ${FD_HOME}/Viola-Jones/ ]] && rm -rf ${FD_HOME}/Viola-Jones/
	[[ -d ${MAIN}/testset/ ]] && rm -rf ${MAIN}/testset/
	[[ -d ${MAIN}/trainset/ ]] && rm -rf ${MAIN}/trainset/
	
	mkdir -p ${MAIN}/
	cd ${FD_HOME}/ && \
		git clone https://github.com/INVASIS/Viola-Jones/ && \
		mv ${FD_HOME}/Viola-Jones/data/testset/ ${MAIN}/ && \
		mv ${FD_HOME}/Viola-Jones/data/trainset/ ${MAIN}/ && \
		rm -rf ${FD_HOME}/Viola-Jones
}


obtainMITDataset() {
	URL="http://cbcl.mit.edu/projects/cbcl/software-datasets/faces.tar.gz"
	cd ${FD_HOME}/data/
    wget "${URL}" || echo -e "An error has occurred whilst trying to download the CMU/MIT dataset."
    DOWNLOADED_FILE="${URL##*/}"
    tar xvzf "${DOWNLOADED_FILE}"
    EXTRACTED_DIR="${DOWNLOADED_FILE%%.*}"
	rm "${DOWNLOADED_FILE}"
    cd - > /dev/null
}


obtainFDDBDataset() {
	URL="http://tamaraberg.com/faceDataset/originalPics.tar.gz"
	cd ${FD_HOME}/data/
    wget "${URL}" || echo -e "An error has occurred whilst trying to download the FDDB dataset."
    DOWNLOADED_FILE="${URL##*/}"
    tar xvzf "${DOWNLOADED_FILE}"
    EXTRACTED_DIR="${DOWNLOADED_FILE%%.*}"
	rm "${DOWNLOADED_FILE}"
    cd - > /dev/null
}


#### BEST FACES DATASET
obtainLabelledFacesInTheWildDataset() {
	URL="http://vis-www.cs.umass.edu/lfw/lfw.tgz"
	cd ${FD_HOME}/data/
    wget "${URL}" || echo -e "An error has occurred whilst trying to download the LFW dataset."
    DOWNLOADED_FILE="${URL##*/}"
    tar xvzf "${DOWNLOADED_FILE}"
    EXTRACTED_DIR="${DOWNLOADED_FILE%%.*}"
	rm "${DOWNLOADED_FILE}"
	mkdir "${FD_HOME}/data/lfw-all/"
	find "${FD_HOME}/data/lfw/" -type f -name "*.jpg" -print | \
	while IFS= read -r file
	do
		mv -v "${file}" "${FD_HOME}/data/lfw-all/"
	done
	rm -rf "${FD_HOME}/data/lfw/"
    cd - > /dev/null
}

#### BEST NON-FACES DATASET
collateAllNonFaces(){
	if [[ -d ${FD_HOME}/data/all-non-faces/ ]]
	then
		rm -rf ${FD_HOME}/data/all-non-faces/
	fi
	
	mkdir -p ${FD_HOME}/data/all-non-faces/

	cp -rv ${FD_HOME}/data/alt/neg/ ${FD_HOME}/data/all-non-faces/
	cp -rv ${FD_HOME}/data/main/testset/non-faces/ ${FD_HOME}/data/all-non-faces/
	cp -rv ${FD_HOME}/data/main/trainset/non-faces/ ${FD_HOME}/data/all-non-faces/
}

openCVPreTrained() {
	if [[ -d ${FD_HOME}/data/haarcascades/ ]]
	then
		rm -rf ${FD_HOME}/data/haarcascades/
	fi
	
	mkdir -p ${FD_HOME}/data/haarcascades/
	
	git clone https://github.com/opencv/opencv/
	mv opencv/data/haarcascades/ ${FD_HOME}/data/
	rm -rf opencv/
}


main() {
	setupWD
	checkPackages
	obtainDatasetMain
	obtainDatasetAlt
	obtainMITDataset
	obtainFDDBDataset
	obtainLabelledFacesInTheWildDataset
	openCVPreTrained
	collateAllNonFaces
}

if [[ "$(whoami)" == "jakeireland" && "$(uname -s)" == "Darwin" ]]
then
	obtainDatasetMain
	obtainDatasetAlt
	openCVPreTrained
	obtainLabelledFacesInTheWildDataset
	collateAllNonFaces
else
	main
fi
